unit loadsaveconfig;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, IniFiles, Dialogs;

const

  defaultDeviceAlias = 'Device';
  defaultDeviceCount = 16;
  defaultChannelCount = 33;


type
  TScaleOffsetArray = array [0..defaultChannelCount-1] of single;

  TDAQCardSettings = record
    DAQCardAlias:string;
    NumberOfActiveAiChannels:byte;
    SampleRate: double;
    SamplesPerChannel:longint;
  end;

  TCalibrationSettings = record
    NumberOfAnalogChannels : byte;
    Scales : array of single;
    Offsets : array of single;
  end;
var
  i: integer;

{device.ini}

//create default device.ini
procedure CreateDefaultDeviceIni;

//load default alias of devices to TStringGrid collumn
//from file device.ini
procedure LoadDefaultDeviceAliasesToGridCol(Col:TStrings);

//Save new values of alias of devices to device.ini
procedure SaveNewDeviceAliasesFromGridCol(Col:TStrings);

{daqcard.ini}

//create/save daq card ini file
procedure CreateDAQCardIni(ADAQCardSettings:TDAQCardSettings);
//load daq card settings from daqcard.ini
procedure LoadDAQCardIni;

{calibration.ini}
procedure CreateCalibrationIni(const ACalibrationScales,ACalibrationOffsets:array of single);
procedure LoadCalibrationIni;

var
  DeviceCount : byte = 16;
  DeviceCheckedList: array of boolean;
  ActiveChannelCount : word;

  CalibrationScales : TScaleOffsetArray =
    (1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,
     50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,
     1000);

  CalibrationOffsets : TScaleOffsetArray =
    (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     0);


  //init default values
  DAQCardSettings: TDAQCardSettings = (
    DAQCardAlias: 'Dev1';
    NumberOfActiveAiChannels:defaultChannelCount;
    SampleRate: 10000;
    SamplesPerChannel: 1000;
  );


implementation

procedure CreateDefaultDeviceIni;
var
  IniFile:TIniFile;
  i:integer;
begin
  if NOT(FileExists(GetAppConfigDir(False)+'device.ini')) then begin
     IniFile:=TIniFile.Create(GetAppConfigDir(False)+'device.ini');
     try
       for i:=1 to DeviceCount do IniFile.WriteString('AliasOfDevices',IntToStr(i),defaultDeviceAlias+IntToStr(i));
     finally
       IniFile.Free;
     end;
  end;
end;

procedure LoadDefaultDeviceAliasesToGridCol(Col:TStrings);
var
  IniFile:TIniFile;
  AStrings:TStrings;
  i:integer;
begin
  CreateDefaultDeviceIni;
  IniFile:=TIniFile.Create(GetAppConfigDir(False)+'device.ini');
  AStrings:=TStringList.Create;
  try
    IniFile.ReadSectionValues('AliasOfDevices',AStrings);
    //dummy string for fixed cell
    AStrings.Insert(0,'');
    for i:=0 to AStrings.Count-1 do Col.add(AStrings.ValueFromIndex[i]);
  finally
    Col.EndUpdate;
    AStrings.Free;
    IniFile.Free;
  end;
end;

procedure SaveNewDeviceAliasesFromGridCol(Col: TStrings);
var
  i:integer;
  IniFile:TIniFile;
begin
  IniFile:=TIniFile.Create(GetAppConfigDir(False)+'device.ini');
  try
    IniFile.EraseSection('AliasOfDevices');
    for i:=1 to Col.Count-1 do IniFile.WriteString('AliasOfDevices',IntToStr(i),Col.Strings[i]);
  finally
    IniFile.Free;
  end;
end;

procedure CreateDAQCardIni(ADAQCardSettings: TDAQCardSettings);
var
  IniFile:TIniFile;
begin
  IniFile:=TIniFile.Create(GetAppConfigDir(False)+'daqcard.ini');
  try
    IniFile.EraseSection('DAQCard');
    IniFile.WriteString('DAQCard','Alias',ADAQCardSettings.DAQCardAlias);
    IniFile.WriteInteger('DAQCard','NumberOfActiveAiChannels',ADAQCardSettings.NumberOfActiveAiChannels);
    IniFile.WriteFloat('DAQCard','SampleRate',ADAQCardSettings.SampleRate);
    IniFile.WriteInteger('DAQCard','SamplesPerChannel',ADAQCardSettings.SamplesPerChannel);
  finally
    IniFile.Free;
  end;
end;


procedure LoadDAQCardIni;
var
  IniFile:TIniFile;
begin
  if NOT(FileExists(GetAppConfigDir(False)+'daqcard.ini')) then CreateDAQCardIni(DAQCardSettings);
  IniFile:=TIniFile.Create(GetAppConfigDir(False)+'daqcard.ini');
  try
    DAQCardSettings.DAQCardAlias := IniFile.ReadString('DAQCard','Alias',DAQCardSettings.DAQCardAlias);
    DAQCardSettings.NumberOfActiveAiChannels := IniFile.ReadInteger('DAQCard','NumberOfActiveAiChannels',DAQCardSettings.NumberOfActiveAiChannels);
    DAQCardSettings.SampleRate := IniFile.ReadFloat('DAQCard','SampleRate',DAQCardSettings.SampleRate);
    DAQCardSettings.SamplesPerChannel := IniFile.ReadInteger('DAQCard','SamplesPerChannel',DAQCardSettings.SamplesPerChannel);
  finally
    IniFile.Free;
  end;
end;

procedure CreateCalibrationIni(const ACalibrationScales, ACalibrationOffsets: array of single);
var
  i:integer;
  IniFile:TIniFile;
begin
  IniFile:=TIniFile.Create(GetAppConfigDir(False)+'calibration.ini');
  try
    IniFile.EraseSection('ChannelScales');
    for i:=Low(ACalibrationScales) to High(ACalibrationScales) do IniFile.WriteFloat('CalibrationScales','Channel'+IntToStr(i),ACalibrationScales[i]);
    for i:=Low(ACalibrationOffsets) to High(ACalibrationOffsets) do IniFile.WriteFloat('CalibrationOffsets','Channel'+IntToStr(i),ACalibrationOffsets[i]);
  finally
    IniFile.Free;
  end;
end;

procedure LoadCalibrationIni;
var
  i:integer;
  IniFile:TIniFile;
begin
  if NOT(FileExists(GetAppConfigDir(False)+'calibration.ini')) then CreateCalibrationIni(CalibrationScales,CalibrationOffsets);
  IniFile:=TIniFile.Create(GetAppConfigDir(False)+'calibration.ini');
  try
    for i:=Low(CalibrationScales) to High(CalibrationScales) do CalibrationScales[i] := IniFile.ReadFloat('CalibrationScales','Channel'+IntToStr(i),CalibrationScales[i]);
    for i:=Low(CalibrationOffsets) to High(CalibrationOffsets) do CalibrationOffsets[i] := IniFile.ReadFloat('CalibrationOffsets','Channel'+IntToStr(i),CalibrationOffsets[i]);
  finally
    IniFile.Free;
  end;
end;

initialization
  SetLength(DeviceCheckedList, DeviceCount);
  for i:= Low(DeviceCheckedList) to High(DeviceCheckedList) do DeviceCheckedList[i] := True;

end.

