unit measfile;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, dialogs, StrUtils, dateutils, Grids, TAGraph, TASeries,
  loadsaveconfig, ComCtrls, StdCtrls, Forms, FileUtil,syncobjs, TAChartUtils;

type

  { TReadMeasFileThread }

  TReadMeasFileThread = class(TThread)
  private
    FFileBufferSize: longword;
    FFileName: string;
    FIbChart: TChart;
    FIbGrid: TStringGrid;
    FIrChart: TChart;
    FIrGrid: TStringGrid;
    FMemo: TMemo;
    FProgressBar: TProgressBar;
    FReadedBytes:int64;
    FStatusPanel: TStatusPanel;

    FTemperatureChart: TChart;
    FTemperatureGrid: TStringGrid;
    FVoltageChart: TChart;
    FVoltageGrid: TStringGrid;
    FFileSize: int64;
    FAString: string;
    FRowN: integer;
    FileBuffer: array of string;

    procedure SetFileBufferSize(AValue: longword);
    procedure SetFileName(AValue: string);
    procedure SetIbChart(AValue: TChart);
    procedure SetIbGrid(AValue: TStringGrid);
    procedure SetIrChart(AValue: TChart);
    procedure SetIrGrid(AValue: TStringGrid);
    procedure SetMemo(AValue: TMemo);
    procedure SetProgressBar(AValue: TProgressBar);
    procedure SetStatusPanel(AValue: TStatusPanel);

    procedure SetTemperatureChart(AValue: TChart);
    procedure SetTemperatureGrid(AValue: TStringGrid);
    procedure SetVoltageChart(AValue: TChart);
    procedure SetVoltageGrid(AValue: TStringGrid);
    procedure UpdateProgress;
    procedure UpdateGrid;
    procedure ReadStrings;
    procedure ReadStrings1;
  protected
    procedure Execute; override;
  public
    cs:TCriticalSection;
    property IrGrid          : TStringGrid read FIrGrid write SetIrGrid;
    property IbGrid          : TStringGrid read FIbGrid write SetIbGrid;
    property TemperatureGrid : TStringGrid read FTemperatureGrid write SetTemperatureGrid;
    property VoltageGrid     : TStringGrid read FVoltageGrid write SetVoltageGrid;
    property IrChart           : TChart read FIrChart write SetIrChart;
    property IbChart           : TChart read FIbChart write SetIbChart;
    property TemperatureChart  : TChart read FTemperatureChart write SetTemperatureChart;
    property VoltageChart      : TChart read FVoltageChart write SetVoltageChart;
    property ProgressBar: TProgressBar read FProgressBar write SetProgressBar;
    property StatusPanel: TStatusPanel read FStatusPanel write SetStatusPanel;

    property FileName: string read FFileName write SetFileName;
    property FileBufferSize: longword read FFileBufferSize write SetFileBufferSize;
    property Memo:TMemo read FMemo write SetMemo;
    constructor Create(CreateSuspended: boolean);
    destructor Destroy;override;
  end;


procedure CreateMeasFileHeader(FileName:string; StartDate,EndDate:TDateTime;
    Note:string; StabilityType:string);
//vraci pocet prectenych bytu
function ReadMeasFileHeader(const FileName:string; out StartDate,EndDate:TDateTime;
    out Note:string; out StabilityType:string): longword;

function GetFileHeaderSize(const FileName: string): int64;
function GetTableHeaderSize(const FileName: string): int64;

function GetFileNumberOfRecords(const FileName: string): single;

procedure CreateMeasFileTableTitles(const FileName:string; Titles: TStrings);

function ReadMeasFileTableTitles(const FileName:string; Titles: TStrings): longword;

procedure ReadMeasFileTableTitlesToCharts(const FileName:string; _IrChart, _IbChart, _TemperatureChart: TChart);

procedure WriteDataToMeasFile(const FileName: string;
    const IrArr, IbArr, TArr: array of double; const Ur, Ub: double);

//vraci pocet prectenych bytu
function ReadDataFromMeasFileToGrids(const FileName: string;
    const _IrGrid, _IbGrid, _TemperatureGrid, _VoltageGrid :TStringGrid;
    const _IrChart, _IbChart, _TemperatureChart, _VoltageChart: TChart): longword;

procedure WriteDataToMeasFileFromGrids(const FileName: string;
    const _IrGrid, _IbGrid, _TemperatureGrid, _VoltageGrid :TStringGrid);

function SaveWaveforms(const Chart: TChart; const FileName: string): boolean;

resourcestring
  FileOpeningInProgressStr = 'File opening in progress...';
const
  DateStr = 'Date';
  TimeStr = 'Time';
  IrStr = 'Ir, [mA]';
  IdStr = 'Id, [mA]';
  UrStr = 'Ur, [V]';
  UdStr = 'Ud, [V]';
  TemperatureStr = 'T, [°C]';
  StartTestStr = 'Test started';
  EndTestStr = 'Test ended';
  NoteStr = 'Note';
  StabilityStr = 'Stability';


implementation

procedure CreateMeasFileHeader(FileName: string; StartDate, EndDate:TDateTime;
  Note: string; StabilityType: string);
var
  FileHeader:TStringList;
begin
  FileHeader :=TStringList.Create;
  try
    FileHeader.Add(StartTestStr+#9+DateTimeToStr(StartDate));
    FileHeader.Add(EndTestStr+#9+DateTimeToStr(EndDate));
    FileHeader.Add(NoteStr+#9+Note);
    FileHeader.Add(StabilityStr+#9+StabilityType);
    FileHeader.SaveToFile(FileName);
  finally
    FileHeader.Free;
  end;
end;

function ReadMeasFileHeader(const FileName: string; out StartDate,
  EndDate: TDateTime; out Note: string; out StabilityType: string): longword;
var

  AFile: TextFile;
  AString: string;
begin
  Result := 0;

  assignFile(AFile, FileName);
  try
    reset(AFile);

    readln(AFile, AString);
    Result := Result + Length(AString);
    StartDate := StrToDateTime(ExtractDelimited(2,AString,[#9]));


    readln(AFile, AString);
    EndDate := StrToDateTime(ExtractDelimited(2,AString,[#9]));
    Result := Result + Length(AString);

    readln(AFile, AString);
    Note := ExtractDelimited(2,AString,[#9]);
    Result := Result + Length(AString);

    readln(AFile, AString);
    StabilityType := ExtractDelimited(2,AString,[#9]);
    Result := Result + Length(AString);
  finally
    CloseFile(AFile);
  end;
end;

function GetFileHeaderSize(const FileName: string): int64;
var
  FileStream: TFileStream;
  C: Char;
  LineCount: byte = 0;
begin
   Result := 0;
   FileStream := TFileStream.Create(FileName,fmOpenRead);
   try
     while NOT(LineCount = 4) do begin
       FileStream.Read(C, SizeOf(C));
       Result := Result + SizeOf(C);
       if C = sLineBreak then inc(LineCount);
       if Result > 10000 then begin
         Result := -1;
         break;
       end;
     end;
   finally
     FreeAndNil(FileStream);
   end;
end;

function GetTableHeaderSize(const FileName: string): int64;
var
  FileStream: TFileStream;
  C: Char;
  LineCount: byte = 0;
begin
   Result := 0;
   FileStream := TFileStream.Create(FileName,fmOpenRead);
   try
     while NOT(LineCount = 5) do begin
       FileStream.Read(C, SizeOf(C));
       if LineCount = 4 then Result := Result + SizeOf(C);
       if C = sLineBreak then inc(LineCount);
       if Result > 10000 then begin
         Result := -1;
         break;
       end;
     end;
   finally
     FreeAndNil(FileStream);
   end;
end;

function GetFileNumberOfRecords(const FileName: string): single;
var
  FileStream: TFileStream;
  C: Char;
  LineCount: byte = 0;
  RecordSize: word = 0;
  FileHeaderSize, TableHeaderSize: int64;
begin
   Result := 0;
   FileHeaderSize := GetFileHeaderSize(FileName);
   TableHeaderSize := GetTableHeaderSize(FileName);
   FileStream := TFileStream.Create(FileName,fmOpenRead);
   try
     RecordSize := 0;
     while NOT(LineCount = 6) do begin
       FileStream.Read(C, SizeOf(C));
       if C = sLineBreak then inc(LineCount);
       if LineCount = 5 then inc(RecordSize);
     end;
     Result := (FileSize(FileName)- FileHeaderSize - TableHeaderSize)/RecordSize;
   finally
     FreeAndNil(FileStream);
   end;
end;

procedure CreateMeasFileTableTitles(const FileName: string; Titles: TStrings);
var
  AFile: TextFile;
  i: integer;
begin
  assignFile(AFile, FileName);
  try
    append(AFile);
    write(AFile,DateStr + #9 + TimeStr + #9);
    for i:= 1 to Titles.Count-1 do begin
      write(AFile, Titles.Strings[i] + ' ' + IrStr + #9);
      write(AFile, Titles.Strings[i] + ' ' + IdStr + #9);
      write(AFile, Titles.Strings[i] + ' ' + TemperatureStr + #9);
    end;
    write(AFile,UrStr + #9 + UdStr);
  finally
    closeFile(AFile);
  end;
end;

function ReadMeasFileTableTitles(const FileName: string; Titles: TStrings
  ): longword;
var
  AFile: TextFile;
  AString: string;
  AStringList: TStringList;
  i: integer;
begin
  Result:= 0;
  assignFile(AFile,FileName);
  AStringList := TStringList.Create;
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile,AString);
    Result := Length(AString);

    AString := StringReplace(AString,DateStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,TimeStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,IrStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,IdStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,TemperatureStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,UrStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,UdStr,'',[rfReplaceAll]);
    for i:= 1 to WordCount(AString,[#9]) do AStringList.Add(ExtractDelimited(i,AString,[#9]));
    //remove duplicates
    for i:= AStringList.Count-1 downto 1 do
      if AStringList.Strings[i] = AStringList.Strings[i-1] then AStringList.Delete(i);
    //add dummy for Col title
    AStringList.Insert(0,'');
    Titles.Assign(AStringList);

  finally
    AStringList.Create;
    closeFile(AFile);
  end;
end;

procedure ReadMeasFileTableTitlesToCharts(const FileName: string; _IrChart, _IbChart,
  _TemperatureChart: TChart);
var
  AFile: TextFile;
  AString: string;
  AStringList: TStringList;
  i: integer;
begin
  assignFile(AFile,FileName);
  AStringList := TStringList.Create;
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile,AString);

    AString := StringReplace(AString,DateStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,TimeStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,IrStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,IdStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,TemperatureStr,'',[rfReplaceAll]);
    AString := StringReplace(AString,UrStr+#9,'',[rfReplaceAll]);
    AString := StringReplace(AString,UdStr,'',[rfReplaceAll]);
    for i:= 1 to WordCount(AString,[#9]) do AStringList.Add(ExtractDelimited(i,AString,[#9]));
    //remove duplicates
    for i:= AStringList.Count-1 downto 1 do
      if AStringList.Strings[i] = AStringList.Strings[i-1] then AStringList.Delete(i);

    for i:= 0 to AStringList.Count-1 do begin
      TLineSeries(_IrChart.Series.Items[i]).Title := AStringList.Strings[i];
      TLineSeries(_IbChart.Series.Items[i]).Title := AStringList.Strings[i];
      TLineSeries(_TemperatureChart.Series.Items[i]).Title := AStringList.Strings[i];
    end;
  finally
    AStringList.Create;
    closeFile(AFile);
  end;
end;

procedure WriteDataToMeasFile(const FileName: string;
  const IrArr, IbArr, TArr: array of double; const Ur, Ub: double);
var
  i: integer;
  AFile: TextFile;
begin
  assignFile(AFile, FileName);
  try
    append(AFile);
    writeln(AFile);
    write(AFile,DateToStr(DateOf(Date)) + #9 + TimeToStr(Time) + #9);
    for i:= Low(IrArr) to High(IrArr) do
      write(AFile, FloatToStrF(IrArr[i],ffFixed,8,2) + #9 +
        FloatToStrF(IbArr[i],ffFixed,8,2) + #9 + FloatToStrF(TArr[i],ffFixed,8,0) + #9);
    write(AFile, FloatToStrF(Ur,ffFixed,8,0) + #9 + FloatToStrF(Ub,ffFixed,8,0));
  finally
    CloseFile(AFile);
  end;
end;

function ReadDataFromMeasFileToGrids(const FileName: string; const _IrGrid,
  _IbGrid, _TemperatureGrid, _VoltageGrid: TStringGrid; const _IrChart,
  _IbChart, _TemperatureChart, _VoltageChart: TChart): longword;
var
  AFile: TextFile;
  AString: string;
  i: integer;
  RowN: integer = 0;
begin
  Result := 0;
  for i:= 0 to _IrChart.SeriesCount-1 do TLineSeries(_IrChart.Series.Items[i]).Clear;
  for i:= 0 to _IbChart.SeriesCount-1 do TLineSeries(_IbChart.Series.Items[i]).Clear;
  for i:= 0 to _TemperatureChart.SeriesCount-1 do TLineSeries(_TemperatureChart.Series.Items[i]).Clear;
  for i:= 0 to _VoltageChart.SeriesCount-1 do TLineSeries(_VoltageChart.Series.Items[i]).Clear;

  assignFile(AFile, FileName);
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    while NOT(EOF(AFile)) do begin
      inc(RowN);
      readln(AFile, AString);
      _IrGrid.RowCount := _IrGrid.RowCount + 1;
      _IbGrid.RowCount := _IbGrid.RowCount + 1;
      _TemperatureGrid.RowCount := _TemperatureGrid.RowCount + 1;
      _VoltageGrid.RowCount := _VoltageGrid.RowCount + 1;

      _IrGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _IrGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      _IbGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _IbGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      _TemperatureGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _TemperatureGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      _VoltageGrid.Cells[1,RowN] := ExtractDelimited(1,AString,[#9]);
      _VoltageGrid.Cells[2,RowN] := ExtractDelimited(2,AString,[#9]);
      for i:= 1 to defaultDeviceCount do begin
        _IrGrid.Cells[i+2,RowN] := ExtractDelimited(i*3, AString,[#9]);
        _IbGrid.Cells[i+2,RowN] := ExtractDelimited(i*3+1, AString,[#9]);
        _TemperatureGrid.Cells[i+2,RowN] := ExtractDelimited(i*3+2, AString,[#9]);

        TLineSeries(_IrChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(i*3, AString,[#9])));
        TLineSeries(_IbChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(i*3+1, AString,[#9])));
        TLineSeries(_TemperatureChart.Series.Items[i-1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(i*3+2, AString,[#9])));

      end;
      _VoltageGrid.Cells[3,RowN] := ExtractDelimited(51, AString,[#9]);
      _VoltageGrid.Cells[4,RowN] := ExtractDelimited(52, AString,[#9]);
      TLineSeries(_VoltageChart.Series.Items[0]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(51, AString,[#9])));
      TLineSeries(_VoltageChart.Series.Items[1]).AddXY(StrToDateTime(ExtractDelimited(1,AString,[#9])+' '+ExtractDelimited(2,AString,[#9])),StrToFloat(ExtractDelimited(52, AString,[#9])));

      Result := Result + Length(AString);


    end;
  finally
    CloseFile(AFile);
  end;
end;

procedure WriteDataToMeasFileFromGrids(const FileName: string; const _IrGrid,
  _IbGrid, _TemperatureGrid, _VoltageGrid: TStringGrid);
var
  i,j: integer;
  AFile:TextFile;
begin
  assignFile(AFile, FileName);
  try
    append(AFile);
    for i:= 1 to _IrGrid.RowCount-1 do begin
      writeln(AFile);
      //date nad time
      for j:= 1 to 2 do
          write(AFile, _IrGrid.Cells[j,i] + #9);
      //currents Ir,Ib and temperature value
      for j:= 3 to _IrGrid.ColCount-1 do
        write(AFile, _IrGrid.Cells[j,i] + #9 + _IbGrid.Cells[j,i] + #9 + _TemperatureGrid.Cells[j,i] + #9);
      //voltage
      write(AFile,_VoltageGrid.Cells[3,i] + #9);
      write(AFile,_VoltageGrid.Cells[4,i]);
    end;
  finally
    CloseFile(AFile);
  end;
end;

function SaveWaveforms(const Chart: TChart; const FileName: string): boolean;
var
  AFile: Text;
  i,j: integer;
begin
  assignFile(AFile, FileName);
  try
    rewrite(AFile);
    //header
    write(AFile, format('%s,[ms]' + #9,['Time']));
    for i:= 1 to 16 do write(AFile,format('%s%d,[mA]' + #9,['Ir',i]));
    writeln(AFile, format('%s,[V]',['U']));
    for j:= 0 to TLineSeries(Chart.Series.Items[0]).Count - 1 do begin
      write(AFile,format('%5.1f' + #9,[TLineSeries(Chart.Series.Items[0]).GetXValue(j)]));
      for i:= 0 to 15 do write(AFile,format('%5.1f' + #9, [TLineSeries(Chart.Series.Items[i]).GetYValue(j)]));
      writeln(AFile,format('%5.1f', [TLineSeries(Chart.Series.Items[16]).GetYValue(j)]));

    end;

  finally
    CloseFile(AFile);
  end;
end;

{ TReadMeasFileThread }

procedure TReadMeasFileThread.SetProgressBar(AValue: TProgressBar);
begin
  if FProgressBar=AValue then Exit;
  FProgressBar:=AValue;
  FProgressBar.Visible := True;
end;

procedure TReadMeasFileThread.SetStatusPanel(AValue: TStatusPanel);
begin
  if FStatusPanel=AValue then Exit;
  FStatusPanel:=AValue;
end;

procedure TReadMeasFileThread.SetTemperatureChart(AValue: TChart);
begin
  if FTemperatureChart=AValue then Exit;
  FTemperatureChart:=AValue;
end;

procedure TReadMeasFileThread.SetTemperatureGrid(AValue: TStringGrid);
begin
  if FTemperatureGrid=AValue then Exit;
  FTemperatureGrid:=AValue;
end;

procedure TReadMeasFileThread.SetVoltageChart(AValue: TChart);
begin
  if FVoltageChart=AValue then Exit;
  FVoltageChart:=AValue;
end;

procedure TReadMeasFileThread.SetVoltageGrid(AValue: TStringGrid);
begin
  if FVoltageGrid=AValue then Exit;
  FVoltageGrid:=AValue;
end;

procedure TReadMeasFileThread.UpdateProgress;
begin
   StatusPanel.Text := FileOpeningInProgressStr;
   if ProgressBar <> Nil then ProgressBar.Position := Round(FReadedBytes/FFileSize * 100);
end;

procedure TReadMeasFileThread.UpdateGrid;
var
  i,j: integer;
  Buffer: array [1..52] of string;
begin
  cs.Enter;

   IrGrid.RowCount := IrGrid.RowCount + Length(FileBuffer);
   IbGrid.RowCount := IbGrid.RowCount + Length(FileBuffer);
   TemperatureGrid.RowCount := TemperatureGrid.RowCount + Length(FileBuffer);
   VoltageGrid.RowCount := VoltageGrid.RowCount + Length(FileBuffer);

   for j:=  Low(FileBuffer) to High(FileBuffer) do begin
     for i:= 1 to 52 do Buffer[i] :=  ExtractDelimited(i,FileBuffer[j],[#9]);
     IrGrid.Cells[1,FRowN+j] := Buffer[1];
     IrGrid.Cells[2,FRowN+j] := Buffer[2];
     IbGrid.Cells[1,FRowN+j] := Buffer[1];
     IbGrid.Cells[2,FRowN+j] := Buffer[2];
     TemperatureGrid.Cells[1,FRowN+j] := Buffer[1];
     TemperatureGrid.Cells[2,FRowN+j] := Buffer[2];
     VoltageGrid.Cells[1,FRowN+j] :=     Buffer[1];
     VoltageGrid.Cells[2,FRowN+j] :=     Buffer[2];

     for i:= 1 to defaultDeviceCount do begin
       IrGrid.Cells[i+2,FRowN+j] :=         Buffer[i*3];
       IbGrid.Cells[i+2,FRowN+j] :=         Buffer[i*3+1];
       TemperatureGrid.Cells[i+2,FRowN+j]:= Buffer[i*3+2];
       TLineSeries(IrChart.Series.Items[i-1]).AddXY(StrToDateTime(Buffer[1]+ ' ' +Buffer[2]),StrToFloat(Buffer[i*3]));
       TLineSeries(IbChart.Series.Items[i-1]).AddXY(StrToDateTime(Buffer[1]+ ' ' +Buffer[2]),StrToFloat(Buffer[i*3+1]));
       TLineSeries(TemperatureChart.Series.Items[i-1]).AddXY(StrToDateTime(Buffer[1]+ ' ' + Buffer[2]),StrToFloat(Buffer[i*3+2]));
     end;
     VoltageGrid.Cells[3,FRowN+j] := Buffer[51];
     VoltageGrid.Cells[4,FRowN+j] := Buffer[52];
     TLineSeries(VoltageChart.Series.Items[0]).AddXY(StrToDateTime(Buffer[1]+ ' ' +Buffer[2]),StrToFloat(Buffer[51]));
     TLineSeries(VoltageChart.Series.Items[1]).AddXY(StrToDateTime(Buffer[1]+ ' ' +Buffer[2]),StrToFloat(Buffer[52]));

   end;
   FRowN := FRowN + Length(FileBuffer);
 cs.Leave;
end;

procedure TReadMeasFileThread.ReadStrings;
var
  AFile: TextFile;
  i: integer;
  NumberOfReads: integer = 0;
  EveryNRecordsRead: integer = 0;
begin
  i := 0;
  EveryNRecordsRead := Round(GetFileNumberOfRecords(FileName)/6000);
  assignFile(AFile, FileName);
  try
    reset(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    readln(AFile);
    while NOT(EOF(AFile)) do begin
      readln(AFile, FAString);
      FReadedBytes := FReadedBytes + Length(FAString);

      if  EveryNRecordsRead <= 1.1 then begin

        FileBuffer[i] := FAString;
        inc(i);
        if i = FFileBufferSize then begin
          Synchronize(@UpdateGrid);
          Synchronize(@UpdateProgress);
          i := 0;
        end;
      end else begin
       //one of 100 samples reads
       inc(NumberOfReads);

       if NumberOfReads = EveryNRecordsRead then begin
         NumberOfReads := 0;
         FileBuffer[i] := FAString;
         inc(i);
         if i = FFileBufferSize then begin
           Synchronize(@UpdateProgress);
           Synchronize(@UpdateGrid);
           i := 0;
         end;
       end;
     end;

    end;
    if i <> 0  then begin
      SetLength(FileBuffer,i);
      Synchronize(@UpdateGrid);
    end;
  finally
    CloseFile(AFile);
    //Terminate;
  end;
end;

procedure TReadMeasFileThread.ReadStrings1;
var
  FileStream: TFileStream;
  i: integer = 0;
  LineNumber: byte = 0;
  C:Char;
  s: string;
  DataPosition: int64;
begin
  i:=0;
  DataPosition := GetFileHeaderSize(FileName) + GetTableHeaderSize(FileName);
  FileStream :=  TFileStream.Create(FileName,fmOpenRead);
  FileStream.Position := DataPosition;
  try
   Memo.Append(intToStr(FileStream.Size));
   while (FileStream.Position < FileStream.Size) or (i>99) do begin
      while C <> sLineBreak do begin
        FileStream.Read(C,1);
        s := S + C;
        inc(FReadedBytes);
      end;
      Trim(s);
      FileBuffer[i] := s;
      inc(i);

      Synchronize(@UpdateProgress);

      if i = FFileBufferSize then begin
        Synchronize(@UpdateGrid);
        i := 0;
      end;
    end;

  finally
    FreeAndNil(FileStream);
    Terminate;
  end;
end;

procedure TReadMeasFileThread.Execute;
var
  i: integer;
begin
  FReadedBytes := 0;

  for i:= 0 to IrChart.SeriesCount-1 do TLineSeries(IrChart.Series.Items[i]).Clear;
  for i:= 0 to IbChart.SeriesCount-1 do TLineSeries(IbChart.Series.Items[i]).Clear;
  for i:= 0 to TemperatureChart.SeriesCount-1 do TLineSeries(TemperatureChart.Series.Items[i]).Clear;
  for i:= 0 to VoltageChart.SeriesCount-1 do TLineSeries(VoltageChart.Series.Items[i]).Clear;

  IrGrid.BeginUpdate;
  IbGrid.BeginUpdate;
  TemperatureGrid.BeginUpdate;
  VoltageGrid.BeginUpdate;
  IrChart.DisableRedrawing;
  IbChart.DisableRedrawing;
  TemperatureChart.DisableRedrawing;
  VoltageChart.DisableRedrawing;


  ReadStrings;


  VoltageGrid.EndUpdate();
  IrGrid.EndUpdate();
  IbGrid.EndUpdate();
  TemperatureGrid.EndUpdate();
  IrChart.EnableRedrawing;
  IbChart.EnableRedrawing;
  TemperatureChart.EnableRedrawing;
  VoltageChart.EnableRedrawing;
  IrChart.Invalidate;
  StatusPanel.Text := '';

  Terminate;
  //Extent := IrChart.LogicalExtent;
  //Extent.a.X := TLineSeries(IrChart.Series.Items[0]).GetXValue(0);
  //Extent.b.X := TLineSeries(IrChart.Series.Items[0]).GetXValue(12000 div IrChart.Series.Count - 1);
  //Extent.b.Y :=  TLineSeries(IrChart.Series.Items[0]).GetYMax;
  //IrChart.LogicalExtent := Extent;


  //if (TLineSeries(IrChart.Series.Items[0]).Count * IrChart.Series.Count) > 120000 then begin
  //  IrChart.Extent.XMax := TLineSeries(IrChart.Series.Items[0]).GetXValue(12000 div IrChart.Series.Count - 1);
  //  IrChart.Extent.UseXMax := True;
  //end;

end;


procedure TReadMeasFileThread.SetFileName(AValue: string);
begin
  if FFileName=AValue then Exit;
  FFileName:=AValue;
  if FileExists(FFileName) then FFileSize := FileSize(FFileName) - GetFileHeaderSize(FFileName);
end;

procedure TReadMeasFileThread.SetFileBufferSize(AValue: longword);
begin
  if FFileBufferSize=AValue then Exit;
  FFileBufferSize:=AValue;
  SetLength(FileBuffer,FFileBufferSize);
end;

procedure TReadMeasFileThread.SetIbChart(AValue: TChart);
begin
  if FIbChart=AValue then Exit;
  FIbChart:=AValue;
end;

procedure TReadMeasFileThread.SetIbGrid(AValue: TStringGrid);
begin
  if FIbGrid=AValue then Exit;
  FIbGrid:=AValue;
end;

procedure TReadMeasFileThread.SetIrChart(AValue: TChart);
begin
  if FIrChart=AValue then Exit;
  FIrChart:=AValue;
end;

procedure TReadMeasFileThread.SetIrGrid(AValue: TStringGrid);
begin
  if FIrGrid=AValue then Exit;
  FIrGrid:=AValue;
end;

procedure TReadMeasFileThread.SetMemo(AValue: TMemo);
begin
  if FMemo=AValue then Exit;
  FMemo:=AValue;
end;

constructor TReadMeasFileThread.Create(CreateSuspended: boolean);
begin
  inherited Create(CreateSuspended);
  FreeOnTerminate := True;
  cs :=TCriticalSection.Create;
  FRowN := 1;
end;

destructor TReadMeasFileThread.Destroy;
begin
  FProgressBar.Visible := False;
  cs.Free;
  inherited Destroy;
end;


initialization
  FormatSettings.LongDateFormat := 'DD.MM.YYYY';
  FormatSettings.LongTimeFormat := 'hh:nn:ss';
end.

